#!/bin/bash
fail() { [ $# -eq 0 ] || echo "$@"; exit 1; }

# set default umask
umask 022

# Pre-setup: Read the build properties from the previous build
# and discard what we don't want
[ -e build.info ] && cp build.info build_properties
source build_properties


# Load up some libraries
my_dir="$( cd "$( dirname "$0" )" && pwd )"
base_dir=$(dirname ${my_dir})
source "${base_dir}/functions/locker"
source "${base_dir}/functions/common"
source "${base_dir}/functions/retry"

debug() { echo "${@}"; }
fail() { echo "${@}" 2>&1; exit 1;}


# Shuffle stuff around"
[ -e build_properties ] && mv build_properties parent_build_properties
[ -e build.log ] && mv build.log parent_build.log
[ -e "${SUITE}-build.sh" ] && rm "${SUITE}-build.sh"

echo "-------------------"
echo " Distro: ${SUITE}"
echo " Serial: ${SERIAL}"
echo " Type:   ${BUILD_TYPE}"
echo "-------------------"

set -x
# Variables
disk_name="${SUITE}-server-cloudimg-amd64-disk1.img"
raw_name="${PWD}/${SUITE}-server-cloudimg-amd64-disk1.raw"
vhd_name="${PWD}/${SUITE}-server-cloudimg-amd64-disk1.vhd"
disk_root="${DISK_ROOT:-/srv/ec2-images/${SUITE}/${SERIAL}}"
raw_disk="${PWD}/results.raw"
launch_config="${PWD}/launch_config.sh"
register_config="${PWD}/register_config.sh"
pkg_tar="${PWD}/pkg.tar"
pkg_tar_d="${PKG_TAR_D:-${kvm_builder}/azure_pkgs}"
proposed="${PROPOSED:-false}"
vhd_size=${VHD_SIZE:-30}

# Covert image to a RAW disk to work with. The raw image is used
# to populate the daily VHD in Azure
debug "Converting QCow2 to Raw Disk"
qemu-img \
    convert -O raw \
    "${disk_root}/${disk_name}" \
    "${raw_name}" &&
        debug "Converted QCow2 to Raw disk for manipulation" ||
        fail  "Failed to convert QCow2 to Raw disk"

config_opts=(${CONFIG_OPTS})
config_opts+=(
    --version $(${kvm_builder}/ubuntu-adj2version ${SUITE})
    --serial "${SERIAL}"
    --out "${launch_config}"
    )

# Turns on building from proposed
[ "${proposed}" == "true" ] &&
    config_opts+=(--proposed)

# Setup the configuration
${kvm_builder}/azure_config.sh \
    ${config_opts[@]} ||
        fail "Failed to configure instance runtime"

# Full disk populate for 12.04
root_size=2
if [ "${SUITE}" == "precise" ]; then
    root_size=29
    truncate -s 29G "${raw_name}.pre-vhd" &&
        debug "Resized 12.04 image to full size" ||
        fail  "Failed to resize 12.04 to full size"
fi

# Launch KVM to do the work
${kvm_builder}/launch_kvm.sh \
    --id ${BUILD_ID} \
    --user-data "${launch_config}" \
    --cloud-config "${kvm_builder}/config/cloud-azure.cfg" \
    --extra-disk "${raw_name}" \
    --raw-disk "${WORKSPACE}/${SUITE}-output.raw" \
    --raw-size ${root_size} \
    --img-url /srv/builder/images/precise-builder-latest.img ||
        fail "KVM instance failed to build image."

rm "${WORKSPACE}/${SUITE}-output.raw"


# Copy the raw image to make it ready for VHD production
cp  --sparse=always "${raw_name}" "${raw_name}.pre-vhd" &&
    debug "Copied raw image VHD production" ||
    fail "Failed to copy raw image to ${raw_name}.pre-vhd"

# Resize the copied RAW image
debug "Truncating image to ${vhd_size}G"
truncate -s "${vhd_size}G" "${raw_name}.pre-vhd" &&
    debug "Truncated image at ${vhd_size}G" ||
    fail  "Failed to truncate disk image"

# Convert to VHD first, step 1 of cheap hack
# This is a cheap hack...half the time the next command
# will fail with "VERR_INVALID_PARAMETER", so this is the,
# er, workaround
debug "Converting to VHD"
_vbox_cmd convertfromraw --format VHD \
    "${raw_name}.pre-vhd" \
    "${vhd_name}.pre" &&
    debug "Converted raw disk to VHD" ||
    fail "Failed to convert raw image to VHD"

# Clone the disk to fixed, VHD for Azure
debug "Converting to VHD format from raw..."
debug ".....this might take a while...."
_vbox_cmd clonehd --format VHD --variant Fixed \
    "${vhd_name}.pre" \
    "${vhd_name}" &&
    debug "Converted raw disk to VHD format using VirtualBox" ||
    fail  "Failed to convert raw image to VHD disk!"

# Remove the unneeded files
rm "${vhd_name}.pre" "${raw_name}.pre-vhd"

debug "Image Characteristics:"
_vbox_cmd showhdinfo "${vhd_name}"


debug "Raw image converted to VHD"

# Archive the bzip2 file
#debug "Archiving the VHD image"
#pbzip2 -f "${vhd_name}" &&
#    debug "Created archive of the VHD image" ||
#    fail "Failed to compress image"

exit 0
