from __future__ import print_function

import logging
import re
import requests
import subprocess
from debian.changelog import (Changelog)

# http://changelogs.ubuntu.com/changelogs/pool/main/l/linux-meta/linux-meta_4.2.0.17.19/changelog
changelog_url_base = \
    "http://changelogs.ubuntu.com/changelogs/pool/{}/{}/{}/{}_{}/changelog"


class ChangeDelta(object):

    def __init__(self, pkg, changelog=None):
        self._pkg = pkg
        self._changelogs = changelog

    def _set_pkg(self, pkg):
        if pkg is not None:
            self._pkg = str(pkg)

    def _get_pkg(self):
        return self._pkg

    pkg = property(_get_pkg, _set_pkg)

    def _get_changelogs(self):
        try:
            return self._changelogs
        except KeyError:
            return []

    def _set_changelogs(self, changelogs):
        self._changelogs = changelogs

    changelogs = property(_get_changelogs, _set_changelogs)

    def iter_changelogs(self, text=False):
        for block in self.changelogs:
            if text:
                yield str(block)
            else:
                yield block

    def _get_bug_cves(self):
        """extract information from the changelog block"""
        mappings, bugs, cves = ({}, [], [])
        cve_re = re.compile("CVE-\d+-\d+")
        bug_re = re.compile("\(LP:.#(\d+)\)")

        for block in self.iter_changelogs():
            _block = str(block)
            cves_in_block = cve_re.findall(_block)
            cves.extend(cves_in_block)

            bugs_in_block = bug_re.findall(_block)
            bugs.extend(bugs_in_block)

            ver = str(block.version)
            mappings[ver] = {'cves': cves_in_block,
                             'bugs': bugs_in_block}

        self.mappings = mappings
        self.cves = cves
        self.bugs = bugs

    def _get_cves(self):
        self._get_bug_cves()
        return self._cves

    def _set_cves(self, cves):
        self._cves = cves

    cves = property(_get_cves, _set_cves)

    def cve_in_delta(self):
        if len(self.cves) > 0:
            return True
        return False

    def _get_bugs(self):
        self._get_bug_cves()
        return self._bugs

    def _set_bugs(self, bugs):
        self._bugs = bugs

    bugs = property(_get_bugs, _set_bugs)

    def _get_min_version(self):
        if self.changelogs:
            return self.changelogs[-1].version

    min_version = property(_get_min_version)

    def _get_max_version(self):
        if self.changelogs:
            return self.changelogs[0].version

    max_version = property(_get_max_version)

    def _get_mappings(self):
        try:
            return self._mappings
        except KeyError:
            return {}

    def _set_mappings(self, mapping):
        self._mappings = mapping

    mappings = property(_get_mappings, _set_mappings)

    def format_changelogs(self):
        changeblocks = "\n".join(self.iter_changelogs(text=True))
        return changeblocks

    def __str__():
        return self.format_changelogs()


class ReadChangelog(Changelog):

    def __init__(self, pkg, version):
        self.logger = logging.getLogger("__changelog_{}__".format(pkg))
        logging.basicConfig(format=
                        '%(asctime)s  %(levelname)s - [PARSING {}] %(message)s'.format(
                            pkg))
        self.logger.setLevel(logging.DEBUG)
        self.logger.debug("Parsing changelog for {}".format(version))

        ch_url = self.get_changelog_url(pkg, version)
        self.logger.debug("URL: {}".format(ch_url))
        try:
            raw_changelog = self.get_changelog_from_url(ch_url)
            Changelog.__init__(self, raw_changelog)
        except Exception as e:
            self.logger.debug("Failed to parse changelog!\n{}".format(e))

        self.min_version = self._blocks[-1].version
        self.max_version = self.version

        self.logger.debug("Opened changelog:")
        self.logger.debug("   Versions {} through {}".format(self.min_version,
                                                        self.max_version))

    def get_changelog_url(self, pkg, version, pocket='main', url=None):
        """Return the changelog URL"""

        url = url or changelog_url_base
        pdir = pkg[0]
        if pkg.startswith("lib"):
            pdir = pkg[:4]
        return url.format(pocket, pdir, pkg, pkg, version)

    def get_changelog_from_url(self, url):
        """Fetch the change log"""
        try:
            chlog = requests.get(url)
            if chlog.status_code == requests.codes.ok:
                return chlog.text
            else:
                chlog.raise_for_status()

        except requests.exceptions.HTTPError as e:
            self.logger.critical("Failed to fetch changelog at {}:\n{}".format(
                            url, e))

    def compare_versions(self, v1, operator, v2):
        """Dirty, slow hack to compare versions"""
        cmd = ['/usr/bin/dpkg', '--compare-versions', str(v1), str(operator),
                str(v2)]
        try:
            subprocess.check_call(cmd)
        except subprocess.CalledProcessError as e:
            return False

        return True

    def iter_changeblocks(self):
        """Iterate over the change logs"""
        for block in self._blocks:
            yield block

    def get_changes_between(self, minv=None, maxv=None, commits=None):
        """Get the changes between two versions"""
        blocks = []
        # Don't waste CPU time if we are getting the whole log
        if minv is None and maxv is None:
            for block in self.iter_changeblocks():
                blocks.append(block)

        # Now deal with changes between
        minver = minv or self.min_version
        maxver = maxv or self.max_version

        # Allow for comparing the latest version against arbitrary counts
        # i.e. you don't have to know the prior version
        if minver <= -1:
            minver = self.versions[(abs(minver) - 1)]

        if minv or maxv:
            for block in self.iter_changeblocks():
                bver = block.version
                if minv:
                    if not self.compare_versions(bver, 'ge', minver):
                        continue
                    if not maxv:
                        blocks.append(block)
                    elif self.compare_versions(bver, 'le', maxver):
                        blocks.append(block)
                elif maxv:
                    if not self.compare_versions(bver, 'le', maxver):
                        continue
                    if not minv:
                        blocks.append(block)
                    elif self.compare_versions(bver, 'ge', minver):
                        blocks.append(block)

        ret = ChangeDelta(self.package, blocks)
        return ret
